/*
 * Copyright (C) 2010-2021 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#if ENABLE(GPU_PROCESS)
#include "RemoteMediaPlayerManagerProxy.h"

#include "ArgumentCoders.h"
#include "Decoder.h"
#include "HandleMessage.h"
#include "RemoteMediaPlayerConfiguration.h"
#include "RemoteMediaPlayerManagerProxyMessages.h"
#include "RemoteMediaPlayerProxyConfiguration.h"
#include "WebCoreArgumentCoders.h"
#include <WebCore/MediaPlayer.h>
#include <WebCore/MediaPlayerEnums.h>
#include <WebCore/MediaPlayerIdentifier.h>
#include <WebCore/SecurityOriginData.h>
#include <wtf/Vector.h>
#include <wtf/WallTime.h>
#include <wtf/text/WTFString.h>

namespace Messages {

namespace RemoteMediaPlayerManagerProxy {

void CreateMediaPlayer::callReply(IPC::Decoder& decoder, CompletionHandler<void(WebKit::RemoteMediaPlayerConfiguration&&)>&& completionHandler)
{
    Optional<WebKit::RemoteMediaPlayerConfiguration> playerConfiguration;
    decoder >> playerConfiguration;
    if (!playerConfiguration) {
        ASSERT_NOT_REACHED();
        cancelReply(WTFMove(completionHandler));
        return;
    }
    completionHandler(WTFMove(*playerConfiguration));
}

void CreateMediaPlayer::cancelReply(CompletionHandler<void(WebKit::RemoteMediaPlayerConfiguration&&)>&& completionHandler)
{
    completionHandler(IPC::AsyncReplyError<WebKit::RemoteMediaPlayerConfiguration>::create());
}

void CreateMediaPlayer::send(std::unique_ptr<IPC::Encoder>&& encoder, IPC::Connection& connection, const WebKit::RemoteMediaPlayerConfiguration& playerConfiguration)
{
    *encoder << playerConfiguration;
    connection.sendSyncReply(WTFMove(encoder));
}

void GetSupportedTypes::send(std::unique_ptr<IPC::Encoder>&& encoder, IPC::Connection& connection, const Vector<String>& types)
{
    *encoder << types;
    connection.sendSyncReply(WTFMove(encoder));
}

void SupportsTypeAndCodecs::send(std::unique_ptr<IPC::Encoder>&& encoder, IPC::Connection& connection, WebCore::MediaPlayerEnums::SupportsType support)
{
    *encoder << support;
    connection.sendSyncReply(WTFMove(encoder));
}

void OriginsInMediaCache::send(std::unique_ptr<IPC::Encoder>&& encoder, IPC::Connection& connection, const Vector<WebCore::SecurityOriginData>& origins)
{
    *encoder << origins;
    connection.sendSyncReply(WTFMove(encoder));
}

void SupportsKeySystem::send(std::unique_ptr<IPC::Encoder>&& encoder, IPC::Connection& connection, bool supports)
{
    *encoder << supports;
    connection.sendSyncReply(WTFMove(encoder));
}

} // namespace RemoteMediaPlayerManagerProxy

} // namespace Messages

namespace WebKit {

void RemoteMediaPlayerManagerProxy::didReceiveMessage(IPC::Connection& connection, IPC::Decoder& decoder)
{
    if (decoder.messageName() == Messages::RemoteMediaPlayerManagerProxy::CreateMediaPlayer::name()) {
        IPC::handleMessageAsync<Messages::RemoteMediaPlayerManagerProxy::CreateMediaPlayer>(connection, decoder, this, &RemoteMediaPlayerManagerProxy::createMediaPlayer);
        return;
    }
    if (decoder.messageName() == Messages::RemoteMediaPlayerManagerProxy::DeleteMediaPlayer::name()) {
        IPC::handleMessage<Messages::RemoteMediaPlayerManagerProxy::DeleteMediaPlayer>(decoder, this, &RemoteMediaPlayerManagerProxy::deleteMediaPlayer);
        return;
    }
    if (decoder.messageName() == Messages::RemoteMediaPlayerManagerProxy::ClearMediaCache::name()) {
        IPC::handleMessage<Messages::RemoteMediaPlayerManagerProxy::ClearMediaCache>(decoder, this, &RemoteMediaPlayerManagerProxy::clearMediaCache);
        return;
    }
    if (decoder.messageName() == Messages::RemoteMediaPlayerManagerProxy::ClearMediaCacheForOrigins::name()) {
        IPC::handleMessage<Messages::RemoteMediaPlayerManagerProxy::ClearMediaCacheForOrigins>(decoder, this, &RemoteMediaPlayerManagerProxy::clearMediaCacheForOrigins);
        return;
    }
    UNUSED_PARAM(connection);
    UNUSED_PARAM(decoder);
    ASSERT_NOT_REACHED();
}

void RemoteMediaPlayerManagerProxy::didReceiveSyncMessage(IPC::Connection& connection, IPC::Decoder& decoder, std::unique_ptr<IPC::Encoder>& replyEncoder)
{
    if (decoder.messageName() == Messages::RemoteMediaPlayerManagerProxy::GetSupportedTypes::name()) {
        IPC::handleMessageSynchronous<Messages::RemoteMediaPlayerManagerProxy::GetSupportedTypes>(connection, decoder, replyEncoder, this, &RemoteMediaPlayerManagerProxy::getSupportedTypes);
        return;
    }
    if (decoder.messageName() == Messages::RemoteMediaPlayerManagerProxy::SupportsTypeAndCodecs::name()) {
        IPC::handleMessageSynchronous<Messages::RemoteMediaPlayerManagerProxy::SupportsTypeAndCodecs>(connection, decoder, replyEncoder, this, &RemoteMediaPlayerManagerProxy::supportsTypeAndCodecs);
        return;
    }
    if (decoder.messageName() == Messages::RemoteMediaPlayerManagerProxy::OriginsInMediaCache::name()) {
        IPC::handleMessageSynchronous<Messages::RemoteMediaPlayerManagerProxy::OriginsInMediaCache>(connection, decoder, replyEncoder, this, &RemoteMediaPlayerManagerProxy::originsInMediaCache);
        return;
    }
    if (decoder.messageName() == Messages::RemoteMediaPlayerManagerProxy::SupportsKeySystem::name()) {
        IPC::handleMessageSynchronous<Messages::RemoteMediaPlayerManagerProxy::SupportsKeySystem>(connection, decoder, replyEncoder, this, &RemoteMediaPlayerManagerProxy::supportsKeySystem);
        return;
    }
    UNUSED_PARAM(connection);
    UNUSED_PARAM(decoder);
    UNUSED_PARAM(replyEncoder);
    ASSERT_NOT_REACHED();
}

} // namespace WebKit

#endif // ENABLE(GPU_PROCESS)
