/*
 * Copyright (C) 2010-2021 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#if ENABLE(GPU_PROCESS)
#include "RemoteMediaResourceManager.h"

#include "DataReference.h"
#include "Decoder.h"
#include "HandleMessage.h"
#include "RemoteMediaResourceIdentifier.h"
#include "RemoteMediaResourceManagerMessages.h"
#include "WebCoreArgumentCoders.h"
#include <WebCore/FrameLoaderTypes.h>
#include <WebCore/NetworkLoadMetrics.h>
#include <WebCore/ResourceError.h>
#include <WebCore/ResourceRequest.h>
#include <WebCore/ResourceResponse.h>

namespace Messages {

namespace RemoteMediaResourceManager {

void ResponseReceived::callReply(IPC::Decoder& decoder, CompletionHandler<void(WebCore::ShouldContinuePolicyCheck&&)>&& completionHandler)
{
    Optional<WebCore::ShouldContinuePolicyCheck> shouldContinue;
    decoder >> shouldContinue;
    if (!shouldContinue) {
        ASSERT_NOT_REACHED();
        cancelReply(WTFMove(completionHandler));
        return;
    }
    completionHandler(WTFMove(*shouldContinue));
}

void ResponseReceived::cancelReply(CompletionHandler<void(WebCore::ShouldContinuePolicyCheck&&)>&& completionHandler)
{
    completionHandler(IPC::AsyncReplyError<WebCore::ShouldContinuePolicyCheck>::create());
}

void ResponseReceived::send(std::unique_ptr<IPC::Encoder>&& encoder, IPC::Connection& connection, WebCore::ShouldContinuePolicyCheck shouldContinue)
{
    *encoder << shouldContinue;
    connection.sendSyncReply(WTFMove(encoder));
}

void RedirectReceived::callReply(IPC::Decoder& decoder, CompletionHandler<void(WebCore::ResourceRequest&&)>&& completionHandler)
{
    Optional<WebCore::ResourceRequest> returnRequest;
    decoder >> returnRequest;
    if (!returnRequest) {
        ASSERT_NOT_REACHED();
        cancelReply(WTFMove(completionHandler));
        return;
    }
    completionHandler(WTFMove(*returnRequest));
}

void RedirectReceived::cancelReply(CompletionHandler<void(WebCore::ResourceRequest&&)>&& completionHandler)
{
    completionHandler(IPC::AsyncReplyError<WebCore::ResourceRequest>::create());
}

void RedirectReceived::send(std::unique_ptr<IPC::Encoder>&& encoder, IPC::Connection& connection, const WebCore::ResourceRequest& returnRequest)
{
    *encoder << returnRequest;
    connection.sendSyncReply(WTFMove(encoder));
}

} // namespace RemoteMediaResourceManager

} // namespace Messages

namespace WebKit {

void RemoteMediaResourceManager::didReceiveMessage(IPC::Connection& connection, IPC::Decoder& decoder)
{
    if (decoder.messageName() == Messages::RemoteMediaResourceManager::ResponseReceived::name()) {
        IPC::handleMessageAsync<Messages::RemoteMediaResourceManager::ResponseReceived>(connection, decoder, this, &RemoteMediaResourceManager::responseReceived);
        return;
    }
    if (decoder.messageName() == Messages::RemoteMediaResourceManager::RedirectReceived::name()) {
        IPC::handleMessageAsync<Messages::RemoteMediaResourceManager::RedirectReceived>(connection, decoder, this, &RemoteMediaResourceManager::redirectReceived);
        return;
    }
    if (decoder.messageName() == Messages::RemoteMediaResourceManager::DataSent::name()) {
        IPC::handleMessage<Messages::RemoteMediaResourceManager::DataSent>(decoder, this, &RemoteMediaResourceManager::dataSent);
        return;
    }
    if (decoder.messageName() == Messages::RemoteMediaResourceManager::DataReceived::name()) {
        IPC::handleMessage<Messages::RemoteMediaResourceManager::DataReceived>(decoder, this, &RemoteMediaResourceManager::dataReceived);
        return;
    }
    if (decoder.messageName() == Messages::RemoteMediaResourceManager::AccessControlCheckFailed::name()) {
        IPC::handleMessage<Messages::RemoteMediaResourceManager::AccessControlCheckFailed>(decoder, this, &RemoteMediaResourceManager::accessControlCheckFailed);
        return;
    }
    if (decoder.messageName() == Messages::RemoteMediaResourceManager::LoadFailed::name()) {
        IPC::handleMessage<Messages::RemoteMediaResourceManager::LoadFailed>(decoder, this, &RemoteMediaResourceManager::loadFailed);
        return;
    }
    if (decoder.messageName() == Messages::RemoteMediaResourceManager::LoadFinished::name()) {
        IPC::handleMessage<Messages::RemoteMediaResourceManager::LoadFinished>(decoder, this, &RemoteMediaResourceManager::loadFinished);
        return;
    }
    UNUSED_PARAM(connection);
    UNUSED_PARAM(decoder);
    ASSERT_NOT_REACHED();
}

} // namespace WebKit

#endif // ENABLE(GPU_PROCESS)
