/*
 * Copyright (C) 2010-2021 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#if ENABLE(GPU_PROCESS) && ENABLE(MEDIA_SOURCE)
#include "RemoteMediaSourceProxy.h"

#include "ArgumentCoders.h"
#include "Decoder.h"
#include "HandleMessage.h"
#include "RemoteMediaSourceProxyMessages.h"
#include "RemoteSourceBufferIdentifier.h"
#include "WebCoreArgumentCoders.h"
#include <WebCore/ContentType.h>
#include <WebCore/MediaPlayerEnums.h>
#include <WebCore/MediaSourcePrivate.h>
#include <wtf/MediaTime.h>
#include <wtf/Optional.h>

namespace Messages {

namespace RemoteMediaSourceProxy {

void AddSourceBuffer::send(std::unique_ptr<IPC::Encoder>&& encoder, IPC::Connection& connection, WebCore::MediaSourcePrivate::AddStatus status, const Optional<WebKit::RemoteSourceBufferIdentifier>& remoteSourceBufferIdentifier)
{
    *encoder << status;
    *encoder << remoteSourceBufferIdentifier;
    connection.sendSyncReply(WTFMove(encoder));
}

} // namespace RemoteMediaSourceProxy

} // namespace Messages

namespace WebKit {

void RemoteMediaSourceProxy::didReceiveMessage(IPC::Connection& connection, IPC::Decoder& decoder)
{
    if (decoder.messageName() == Messages::RemoteMediaSourceProxy::DurationChanged::name()) {
        IPC::handleMessage<Messages::RemoteMediaSourceProxy::DurationChanged>(decoder, this, &RemoteMediaSourceProxy::durationChanged);
        return;
    }
    if (decoder.messageName() == Messages::RemoteMediaSourceProxy::SetReadyState::name()) {
        IPC::handleMessage<Messages::RemoteMediaSourceProxy::SetReadyState>(decoder, this, &RemoteMediaSourceProxy::setReadyState);
        return;
    }
    if (decoder.messageName() == Messages::RemoteMediaSourceProxy::SetIsSeeking::name()) {
        IPC::handleMessage<Messages::RemoteMediaSourceProxy::SetIsSeeking>(decoder, this, &RemoteMediaSourceProxy::setIsSeeking);
        return;
    }
    if (decoder.messageName() == Messages::RemoteMediaSourceProxy::WaitForSeekCompleted::name()) {
        IPC::handleMessage<Messages::RemoteMediaSourceProxy::WaitForSeekCompleted>(decoder, this, &RemoteMediaSourceProxy::waitForSeekCompleted);
        return;
    }
    if (decoder.messageName() == Messages::RemoteMediaSourceProxy::SeekCompleted::name()) {
        IPC::handleMessage<Messages::RemoteMediaSourceProxy::SeekCompleted>(decoder, this, &RemoteMediaSourceProxy::seekCompleted);
        return;
    }
    if (decoder.messageName() == Messages::RemoteMediaSourceProxy::SetTimeFudgeFactor::name()) {
        IPC::handleMessage<Messages::RemoteMediaSourceProxy::SetTimeFudgeFactor>(decoder, this, &RemoteMediaSourceProxy::setTimeFudgeFactor);
        return;
    }
    UNUSED_PARAM(connection);
    UNUSED_PARAM(decoder);
    ASSERT_NOT_REACHED();
}

void RemoteMediaSourceProxy::didReceiveSyncMessage(IPC::Connection& connection, IPC::Decoder& decoder, std::unique_ptr<IPC::Encoder>& replyEncoder)
{
    if (decoder.messageName() == Messages::RemoteMediaSourceProxy::AddSourceBuffer::name()) {
        IPC::handleMessageSynchronous<Messages::RemoteMediaSourceProxy::AddSourceBuffer>(connection, decoder, replyEncoder, this, &RemoteMediaSourceProxy::addSourceBuffer);
        return;
    }
    UNUSED_PARAM(connection);
    UNUSED_PARAM(decoder);
    UNUSED_PARAM(replyEncoder);
    ASSERT_NOT_REACHED();
}

} // namespace WebKit

#endif // ENABLE(GPU_PROCESS) && ENABLE(MEDIA_SOURCE)
